/**
  * Name: Vue use kit
  * Author: Salvatore Tedde <microcipcip@gmail.com>
  * Website: https://github.com/microcipcip/vue-use-kit 
  * Date: 17/02/2020 
*/
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@vue/composition-api')) :
  typeof define === 'function' && define.amd ? define(['exports', '@vue/composition-api'], factory) :
  (factory((global.vueUseKit = {}),global.vueCompositionApi));
}(this, (function (exports,compositionApi) { 'use strict';

  var compositionApi__default = 'default' in compositionApi ? compositionApi['default'] : compositionApi;

  function getQuery(min, max) {
      if (min === void 0) { min = 0; }
      if (max === void 0) { max = 0; }
      if (!min && !max) {
          throw new Error("Please specify at least one value, either 'min' or 'max'");
      }
      else if (min && !max) {
          return "only screen and (min-width: " + min + "px)";
      }
      else if (!min && max) {
          return "only screen and (max-width: " + (max - 1) + "px)";
      }
      else {
          return "only screen and (min-width: " + min + "px) and (max-width: " + (max - 1) + "px)";
      }
  }

  var defaultEvents = ['mousedown', 'touchstart'];
  function useClickAway(elRef, callback, events) {
      if (events === void 0) { events = defaultEvents; }
      var handler = function (e) {
          if (elRef.value && !elRef.value.contains(e.target)) {
              callback(e);
          }
      };
      compositionApi.onMounted(function () {
          events.forEach(function (evtName) { return document.addEventListener(evtName, handler); });
      });
      compositionApi.onUnmounted(function () {
          events.forEach(function (evtName) { return document.removeEventListener(evtName, handler); });
      });
  }

  function useFullscreen(elRef) {
      var isFullscreen = compositionApi.ref(false);
      var handleFullscreenChange = function () {
          if (!isFullscreen.value)
              return;
          var isCurrentlyFullscreen = !!document.fullscreenElement;
          isFullscreen.value = isCurrentlyFullscreen;
      };
      var startTracking = function () {
          document.addEventListener('fullscreenchange', handleFullscreenChange);
      };
      var stopTracking = function () {
          document.removeEventListener('fullscreenchange', handleFullscreenChange);
      };
      var start = function () {
          if (!elRef.value)
              return;
          if (isFullscreen.value)
              return;
          elRef.value.requestFullscreen().then(function () {
              isFullscreen.value = true;
              startTracking();
          });
      };
      var stop = function () {
          if (!isFullscreen.value)
              return;
          document.exitFullscreen().then(function () {
              isFullscreen.value = false;
              stopTracking();
          });
      };
      compositionApi.onUnmounted(stopTracking);
      return { isFullscreen: isFullscreen, start: start, stop: stop };
  }

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  var defaultOpts = {
      enableHighAccuracy: false,
      timeout: Infinity,
      maximumAge: 0
  };
  function useGeolocation(options, runOnMount) {
      if (options === void 0) { options = {}; }
      if (runOnMount === void 0) { runOnMount = true; }
      options = Object.assign(defaultOpts, options);
      // Note: surprisingly the watchId can be 0 (not positive) so
      // we have to check if watchId !== null every time
      var watchId = null;
      var geoInitData = {
          loading: false,
          accuracy: null,
          altitude: null,
          altitudeAccuracy: null,
          heading: null,
          latitude: null,
          longitude: null,
          speed: null,
          timestamp: null
      };
      var isTracking = compositionApi.ref(false);
      var geo = compositionApi.ref(__assign({}, geoInitData));
      var onEventError = function (error) {
          if (watchId === null)
              return;
          geo.value.loading = false;
          geo.value.error = {
              code: error.code,
              message: error.message
          };
          isTracking.value = false;
      };
      var handleGeolocation = function (_a) {
          var coords = _a.coords, timestamp = _a.timestamp;
          geo.value = {
              loading: false,
              accuracy: coords.accuracy,
              altitude: coords.altitude,
              altitudeAccuracy: coords.altitudeAccuracy,
              heading: coords.heading,
              latitude: coords.latitude,
              longitude: coords.longitude,
              speed: coords.speed,
              timestamp: timestamp
          };
          isTracking.value = true;
      };
      var start = function () {
          if (watchId !== null)
              return;
          geo.value.loading = true;
          geo.value.timestamp = Date.now();
          navigator.geolocation.getCurrentPosition(handleGeolocation, onEventError, options);
          watchId = navigator.geolocation.watchPosition(handleGeolocation, onEventError, options);
      };
      var stop = function () {
          if (watchId === null)
              return;
          navigator.geolocation.clearWatch(watchId);
          watchId = null;
          isTracking.value = false;
      };
      compositionApi.onMounted(function () { return runOnMount && start(); });
      compositionApi.onUnmounted(stop);
      return { isTracking: isTracking, geo: geo, start: start, stop: stop };
  }

  function useHover(elRef) {
      var isHovered = compositionApi.ref(false);
      var mouseEnterHandler = function () {
          isHovered.value = true;
      };
      var mouseLeaveHandler = function () {
          isHovered.value = false;
      };
      compositionApi.onMounted(function () {
          if (!elRef.value)
              return;
          elRef.value.addEventListener('mouseenter', mouseEnterHandler);
          elRef.value.addEventListener('mouseleave', mouseLeaveHandler);
      });
      compositionApi.onUnmounted(function () {
          if (!elRef.value)
              return;
          elRef.value.removeEventListener('mouseenter', mouseEnterHandler);
          elRef.value.removeEventListener('mouseleave', mouseLeaveHandler);
      });
      return isHovered;
  }

  var errorMsg = 'IntersectionObserver is not supported, please install a polyfill';
  function useIntersection(elRef, options, runOnMount) {
      if (options === void 0) { options = {}; }
      if (runOnMount === void 0) { runOnMount = true; }
      var observedEntry = compositionApi.ref(null);
      var handleObserver = function (entries) {
          observedEntry.value = entries[0];
      };
      var observer = null;
      var start = function () {
          if (!('IntersectionObserver' in window))
              throw new Error(errorMsg);
          // Do not start if the observer is already initialized
          // or the elRef does not exist
          if (observer || !elRef.value)
              return;
          observer = new IntersectionObserver(handleObserver, options);
          observer.observe(elRef.value);
      };
      var stop = function () {
          if (!observer)
              return;
          observer.disconnect();
          observer = null;
      };
      compositionApi.onMounted(function () { return runOnMount && start(); });
      compositionApi.onUnmounted(stop);
      return { observedEntry: observedEntry, start: start, stop: stop };
  }

  function useMedia(query, defaultState) {
      if (defaultState === void 0) { defaultState = false; }
      var mql;
      var matches = compositionApi.ref(defaultState);
      var updateMatchValue = function () { return (matches.value = mql.matches); };
      compositionApi.onMounted(function () {
          mql = window.matchMedia(query);
          mql.addListener(updateMatchValue);
          updateMatchValue();
      });
      compositionApi.onUnmounted(function () {
          mql.removeListener(updateMatchValue);
      });
      return matches;
  }

  function useMouse() {
      var docX = compositionApi.ref(0);
      var docY = compositionApi.ref(0);
      var mouseMoveHandler = function (e) {
          docX.value = e.pageX;
          docY.value = e.pageY;
      };
      compositionApi.onMounted(function () {
          document.addEventListener('mousemove', mouseMoveHandler);
      });
      compositionApi.onUnmounted(function () {
          document.removeEventListener('mousemove', mouseMoveHandler);
      });
      return {
          docX: docX,
          docY: docY
      };
  }

  function useMouseElement(elRef) {
      var docX = compositionApi.ref(0);
      var docY = compositionApi.ref(0);
      var elX = compositionApi.ref(0);
      var elY = compositionApi.ref(0);
      var elInfoX = compositionApi.ref(0);
      var elInfoY = compositionApi.ref(0);
      var elInfoW = compositionApi.ref(0);
      var elInfoH = compositionApi.ref(0);
      var mouseMoveHandler = function (e) {
          if (!elRef.value)
              return;
          docX.value = e.pageX;
          docY.value = e.pageY;
          var _a = elRef.value.getBoundingClientRect(), left = _a.left, top = _a.top, height = _a.height, width = _a.width;
          elInfoX.value = left + window.pageXOffset;
          elInfoY.value = top + window.pageYOffset;
          elInfoW.value = width;
          elInfoH.value = height;
          elX.value = e.pageX - elInfoX.value;
          elY.value = e.pageY - elInfoY.value;
      };
      compositionApi.onMounted(function () {
          document.addEventListener('mousemove', mouseMoveHandler);
      });
      compositionApi.onUnmounted(function () {
          document.removeEventListener('mousemove', mouseMoveHandler);
      });
      return {
          docX: docX,
          docY: docY,
          elX: elX,
          elY: elY,
          elInfoX: elInfoX,
          elInfoY: elInfoY,
          elInfoW: elInfoW,
          elInfoH: elInfoH
      };
  }

  function useIntervalFn(callback, ms, runOnMount) {
      if (ms === void 0) { ms = 0; }
      if (runOnMount === void 0) { runOnMount = true; }
      var isRunning = compositionApi.ref(false);
      var interval = null;
      var start = function () {
          if (interval)
              return;
          isRunning.value = true;
          interval = setInterval(callback, ms);
      };
      var stop = function () {
          clearInterval(interval);
          isRunning.value = false;
          interval = null;
      };
      compositionApi.onMounted(function () { return runOnMount && start(); });
      compositionApi.onUnmounted(stop);
      return { isRunning: isRunning, start: start, stop: stop };
  }

  function useInterval(ms, runOnMount) {
      if (ms === void 0) { ms = 0; }
      if (runOnMount === void 0) { runOnMount = true; }
      var counter = compositionApi.ref(0);
      var animHandler = function () {
          counter.value = counter.value + 1;
      };
      var _a = useIntervalFn(animHandler, ms, runOnMount), isRunning = _a.isRunning, start = _a.start, stop = _a.stop;
      return { isRunning: isRunning, counter: counter, start: start, stop: stop };
  }

  var getFps = function (fps) { return Number(compositionApi.isRef(fps) ? fps.value : fps); };
  var calcFpsInterval = function (fps) { return 1000 / fps; };
  var fpsLimit = 60;
  function useRafFn(callback, 
  // Note: a value equal to or greater than 60 will disable the fps check logic
  // giving maximum precision and smoothness
  fps, runOnMount) {
      if (fps === void 0) { fps = fpsLimit; }
      if (runOnMount === void 0) { runOnMount = true; }
      var isRunning = compositionApi.ref(false);
      // Using a computed here allows us to update fps
      // dynamically from user's input
      var fpsInterval = compositionApi.computed(function () { return calcFpsInterval(getFps(fps)); });
      var itWasIdle = false;
      var startTime = 0;
      var timeElapsed = 0;
      var timePausedTot = 0;
      var timeDelta = 0;
      var loop = function (timeStamp) {
          if (!startTime)
              startTime = timeStamp;
          if (!isRunning.value)
              return;
          // Store the time that the loop is idle so that we can
          // calculate the timeElapsed variable correctly and return
          // it to the user from within the callback
          if (itWasIdle) {
              timePausedTot = timeStamp - startTime - timeElapsed;
              itWasIdle = false;
          }
          // Adjust timeElapsed to account for startTime and timePausedTot
          timeElapsed = timeStamp - startTime - timePausedTot;
          // Always run the callback if fps is greater than fpsLimit
          var callbackShouldAlwaysRun = getFps(fps) >= fpsLimit;
          if (callbackShouldAlwaysRun) {
              // Store timeDelta for future computations
              timeDelta = timeElapsed;
              callback(timeElapsed);
          }
          // Run callback only when !callbackShouldAlwaysRun
          // and the given fps matches the lapsed time
          if (!callbackShouldAlwaysRun) {
              var elapsedTimeFromPrevLoop = Math.ceil(timeElapsed - timeDelta);
              if (elapsedTimeFromPrevLoop > fpsInterval.value) {
                  // Store timeDelta for future computations
                  timeDelta = timeElapsed;
                  callback(timeElapsed);
              }
          }
          // Run loop again recursively
          requestAnimationFrame(loop);
      };
      var start = function () {
          isRunning.value = true;
          requestAnimationFrame(loop);
      };
      var stop = function () {
          isRunning.value = false;
          itWasIdle = true;
      };
      compositionApi.onMounted(function () { return runOnMount && start(); });
      compositionApi.onUnmounted(stop);
      return {
          isRunning: isRunning,
          start: start,
          stop: stop
      };
  }

  var fpsLimit$1 = 60;
  function useRaf(fps, runOnMount) {
      if (fps === void 0) { fps = fpsLimit$1; }
      if (runOnMount === void 0) { runOnMount = true; }
      var elapsed = compositionApi.ref(0);
      var animHandler = function (elapsedTime) {
          elapsed.value = elapsedTime;
      };
      var _a = useRafFn(animHandler, fps, runOnMount), isRunning = _a.isRunning, start = _a.start, stop = _a.stop;
      return {
          isRunning: isRunning,
          elapsed: elapsed,
          start: start,
          stop: stop
      };
  }

  function useTimeoutFn(callback, ms, runOnMount) {
      if (ms === void 0) { ms = 0; }
      if (runOnMount === void 0) { runOnMount = true; }
      var isReady = compositionApi.ref(null);
      var timeout = null;
      var start = function () {
          isReady.value = false;
          if (timeout)
              clearTimeout(timeout);
          timeout = setTimeout(function () {
              isReady.value = true;
              callback();
              timeout = null;
          }, ms);
      };
      var stop = function () {
          isReady.value = null;
          if (timeout) {
              clearTimeout(timeout);
              timeout = null;
          }
      };
      compositionApi.onMounted(function () { return runOnMount && start(); });
      compositionApi.onUnmounted(stop);
      return { isReady: isReady, start: start, stop: stop };
  }

  var noop = function () { return null; };
  function useTimeout(ms, runOnMount) {
      if (ms === void 0) { ms = 0; }
      if (runOnMount === void 0) { runOnMount = true; }
      var _a = useTimeoutFn(noop, ms, runOnMount), isReady = _a.isReady, start = _a.start, stop = _a.stop;
      return { isReady: isReady, start: start, stop: stop };
  }

  exports.getQuery = getQuery;
  exports.useClickAway = useClickAway;
  exports.useFullscreen = useFullscreen;
  exports.useGeolocation = useGeolocation;
  exports.useHover = useHover;
  exports.useIntersection = useIntersection;
  exports.useMedia = useMedia;
  exports.useMouse = useMouse;
  exports.useMouseElement = useMouseElement;
  exports.useIntervalFn = useIntervalFn;
  exports.useInterval = useInterval;
  exports.useRafFn = useRafFn;
  exports.useRaf = useRaf;
  exports.useTimeoutFn = useTimeoutFn;
  exports.useTimeout = useTimeout;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
